'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var define = require('./define-124dd882.js');
require('graphql');

function nodeDefinitions(idFetcher) {
    const nodeInterface = define.interfaceType({
        name: 'Node',
        description: 'An object with an ID',
        fields: () => [
            define.abstractField({
                name: 'id',
                type: define.nonnull(define.id),
                description: 'The id of the object.',
            }),
        ],
    });
    const nodeField = define.field({
        name: 'node',
        type: nodeInterface,
        args: {
            id: define.arg({ type: define.nonnullInput(define.id), description: 'The ID of an object' }),
        },
        resolve: (_, { id }, context, info) => idFetcher(id, context, info),
    });
    return { nodeInterface, nodeField };
}
const forwardConnectionArgs = {
    after: define.arg({ type: define.string }),
    first: define.arg({ type: define.int }),
};
const backwardConnectionArgs = {
    before: define.arg({ type: define.string }),
    last: define.arg({ type: define.int }),
};
const connectionArgs = {
    ...forwardConnectionArgs,
    ...backwardConnectionArgs,
};
const pageInfoType = define.objectType({
    name: 'PageInfo',
    description: 'Information about pagination in a connection.',
    fields: () => [
        define.field({
            name: 'hasNextPage',
            type: define.nonnull(define.boolean),
            description: 'When paginating forwards, are there more items?',
        }),
        define.field({
            name: 'hasPreviousPage',
            type: define.nonnull(define.boolean),
            description: 'When paginating backwards, are there more items?',
        }),
        define.field({
            name: 'startCursor',
            type: define.string,
            description: 'When paginating backwards, the cursor to continue.',
        }),
        define.field({
            name: 'endCursor',
            type: define.string,
            description: 'When paginating forwards, the cursor to continue.',
        }),
    ],
});
/**
 * Returns ObjectTypes for a connection with the given name,
 * and whose nodes are of the specified type.
 */
function connectionDefinitions(config) {
    const { nodeType } = config;
    const name = config.name || nodeType.name;
    // TODO
    const edgeFields = config.edgeFields || (() => []);
    const connectionFields = config.connectionFields || (() => []);
    const edgeType = define.objectType({
        name: name + 'Edge',
        description: 'An edge in a connection.',
        fields: () => [
            define.field({
                name: 'node',
                type: define.nonnull(nodeType),
                description: 'The item at the end of the edge',
            }),
            define.field({
                name: 'cursor',
                type: define.nonnull(define.string),
                description: 'A cursor for use in pagination',
            }),
            ...edgeFields(),
        ],
    });
    const connectionType = define.objectType({
        name: name + 'Connection',
        description: 'A connection to a list of items.',
        fields: () => [
            define.field({
                name: 'pageInfo',
                type: define.nonnull(pageInfoType),
                description: 'Information to aid in pagination.',
            }),
            define.field({
                name: 'edges',
                type: define.list(edgeType),
                description: 'A list of edges.',
            }),
            ...connectionFields(),
        ],
    });
    return { edgeType, connectionType };
}

exports.connectionArgs = connectionArgs;
exports.connectionDefinitions = connectionDefinitions;
exports.nodeDefinitions = nodeDefinitions;
