import * as graphql from 'graphql';

function builtInScalar(builtInType) {
    return {
        kind: 'Scalar',
        builtInType,
    };
}
const string = builtInScalar(graphql.GraphQLString);
const int = builtInScalar(graphql.GraphQLInt);
const float = builtInScalar(graphql.GraphQLFloat);
const boolean = builtInScalar(graphql.GraphQLBoolean);
const id = builtInScalar(graphql.GraphQLID);
function scalar({ name, description, serialize, parseValue, parseLiteral, }) {
    return {
        kind: 'Scalar',
        graphqlTypeConfig: {
            name,
            description,
            serialize: serialize,
            parseLiteral,
            parseValue,
        },
    };
}
function enumType({ name, description, values, }) {
    return {
        kind: 'Enum',
        name,
        description,
        values,
    };
}
function arg({ type, description, default: defaultArg, }) {
    return {
        kind: 'Argument',
        type: type,
        description,
        default: defaultArg,
    };
}
function defaultArg(type, defaultArg, description) {
    return {
        kind: 'DefaultArgument',
        type: type,
        description,
        default: defaultArg,
    };
}
function field({ name, type, resolve, args, ...options }) {
    return {
        kind: 'Field',
        name,
        type,
        args: args !== null && args !== void 0 ? args : {},
        // if no resolver is defined we fallback to the default GraphQL resolver which is (src) => src[fieldName]
        resolve: typeof resolve === 'function' ? resolve : undefined,
        ...options,
    };
}
function abstractField(opts) {
    return {
        kind: 'AbstractField',
        name: opts.name,
        description: opts.description,
        deprecationReason: opts.deprecationReason,
        args: opts.args,
        type: opts.type,
    };
}
function objectType({ name, description, interfaces = [], fields, isTypeOf, extensions, }) {
    const obj = {
        kind: 'ObjectType',
        name,
        description,
        interfaces,
        fieldsFn: undefined,
        isTypeOf: isTypeOf,
        extensions,
    };
    obj.fieldsFn = () => fields(obj);
    return obj;
}
function inputObjectType({ name, description, fields, }) {
    let inputObj = {
        kind: 'InputObject',
        name,
        description,
        fieldsFn: null,
    };
    inputObj.fieldsFn = () => fields(inputObj);
    return inputObj;
}
function unionType({ name, description, types, resolveType, }) {
    return {
        kind: 'Union',
        name,
        description,
        types,
        resolveType,
    };
}
function interfaceType({ name, description, interfaces = [], fields, }) {
    const obj = {
        kind: 'Interface',
        name,
        description,
        interfaces,
        fieldsFn: undefined,
    };
    obj.fieldsFn = () => fields(obj);
    return obj;
}
function list(ofType) {
    return {
        kind: 'List',
        ofType: ofType,
    };
}
function listInput(ofType) {
    return {
        kind: 'ListInput',
        ofType: ofType,
    };
}
function nonnull(ofType) {
    return {
        kind: 'NonNull',
        ofType: ofType,
    };
}
function nonnullInput(ofType) {
    return {
        kind: 'NonNullInput',
        ofType: ofType,
    };
}
function queryType({ name = 'Query', fields, }) {
    return {
        kind: 'ObjectType',
        name,
        interfaces: [],
        fieldsFn: fields,
    };
}
function mutationType({ name = 'Mutation', fields, }) {
    return {
        kind: 'ObjectType',
        name,
        interfaces: [],
        fieldsFn: fields,
    };
}
function subscriptionField({ name, type, args = {}, subscribe, description, deprecationReason, }) {
    return {
        kind: 'SubscriptionField',
        name,
        type,
        args,
        subscribe: subscribe,
        resolve: (value) => value,
        description,
        deprecationReason,
    };
}
function subscriptionType({ name = 'Subscription', fields, }) {
    return {
        kind: 'SubscriptionObject',
        name,
        fields,
    };
}

export { id as a, boolean as b, scalar as c, arg as d, enumType as e, float as f, defaultArg as g, field as h, int as i, abstractField as j, inputObjectType as k, interfaceType as l, list as m, listInput as n, objectType as o, nonnull as p, nonnullInput as q, queryType as r, string as s, mutationType as t, unionType as u, subscriptionField as v, subscriptionType as w };
