import * as graphql from 'graphql';
import { Scalar, Enum, EnumValue, ObjectType, InputObject, Interface, Union, InputType, OutputType, ArgMap, TOfArgMap, Field, AbstractField, DefaultArgument, Argument, InputFieldMap, SubscriptionField, SubscriptionObject, PromiseOrValue, Context } from './types';
type ExtensionsMap = {
    field?: {
        [key: string]: any;
    };
    objectType?: {
        [key: string]: any;
    };
};
type ResolvePartialMandatory<Src, Arg, Out> = {
    resolve: (src: Src, args: TOfArgMap<ArgMap<Arg>>, ctx: Context, info: graphql.GraphQLResolveInfo) => PromiseOrValue<Out>;
};
type ResolvePartialOptional<Src, Arg, Out> = {
    resolve?: (src: Src, args: TOfArgMap<ArgMap<Arg>>, ctx: Context, info: graphql.GraphQLResolveInfo) => PromiseOrValue<Out>;
};
export declare const string: Scalar<string | null | undefined>;
export declare const int: Scalar<number | null | undefined>;
export declare const float: Scalar<number | null | undefined>;
export declare const boolean: Scalar<boolean | null | undefined>;
export declare const id: Scalar<string | null | undefined>;
export declare function scalar<Src>({ name, description, serialize, parseValue, parseLiteral, }: {
    name: string;
    description?: string;
    serialize: (src: Src) => any | null;
    parseValue?: (value: unknown) => Src | null;
    parseLiteral?: (value: graphql.ValueNode) => Src | null;
}): Scalar<Src | null>;
export declare function enumType<Src>({ name, description, values, }: {
    name: string;
    description?: string;
    values: Array<EnumValue<Src>>;
}): Enum<Src | null>;
export declare function arg<Src, TDefault extends Exclude<Src, null | undefined> | undefined>({ type, description, default: defaultArg, }: {
    type: InputType<Src>;
    description?: string;
    default?: TDefault;
}): Argument<TDefault extends undefined ? Exclude<Src, undefined> : Exclude<Src, null | undefined>>;
export declare function defaultArg<Src>(type: InputType<Src>, defaultArg: Exclude<Src, null>, description?: string): DefaultArgument<Exclude<Src, null>>;
export declare function field<Key extends string, Src, Out, Arg extends object = {}>({ name, type, resolve, args, ...options }: {
    name: Key;
    type: OutputType<Out>;
    args?: ArgMap<Arg>;
    description?: string;
    deprecationReason?: string;
    extensions?: ExtensionsMap['field'];
} & (Key extends keyof Src ? Src[Key] extends Out ? ResolvePartialOptional<Src, Arg, Out> : ResolvePartialMandatory<Src, Arg, Out> : ResolvePartialMandatory<Src, Arg, Out>)): Field<Src, any, any>;
export declare function abstractField<Out>(opts: {
    name: string;
    type: OutputType<Out>;
    description?: string;
    deprecationReason?: string;
    args?: ArgMap<unknown>;
}): AbstractField<Out>;
export declare function objectType<Src>({ name, description, interfaces, fields, isTypeOf, extensions, }: {
    name: string;
    description?: string;
    interfaces?: Array<Interface<any>>;
    fields: (self: OutputType<Src | null>) => [Field<Src, any, {}>, ...Field<Src, any, {}>[]];
    isTypeOf?: (src: any, ctx: Context, info: graphql.GraphQLResolveInfo) => boolean;
    extensions?: ExtensionsMap['objectType'] extends undefined ? Record<string, any> : ExtensionsMap['objectType'];
}): ObjectType<Src | null>;
export declare function inputObjectType<Src>({ name, description, fields, }: {
    name: string;
    description?: string;
    fields: (self: InputType<Src | null>) => InputFieldMap<Src>;
}): InputObject<Src | null>;
export declare function unionType<Src>({ name, description, types, resolveType, }: {
    name: string;
    description?: string;
    types: Array<ObjectType<any>> | (() => Array<ObjectType<any>>);
    resolveType: (src: Src) => string;
}): Union<Src | null>;
export declare function interfaceType<Src>({ name, description, interfaces, fields, }: {
    name: string;
    description?: string;
    interfaces?: Array<Interface<any>> | (() => Array<Interface<any>>);
    fields: (self: Interface<Src | null>) => Array<AbstractField<any>>;
}): Interface<Src | null>;
export declare function list<Src>(ofType: OutputType<Src>): OutputType<Array<Src> | null>;
export declare function listInput<Src>(ofType: InputType<Src>): InputType<Array<Src> | null>;
export declare function nonnull<Src>(ofType: OutputType<Src | null | undefined>): OutputType<Src>;
export declare function nonnull<Src>(ofType: OutputType<Src | null>): OutputType<Src>;
export declare function nonnullInput<Src>(ofType: InputType<Src | null | undefined>): InputType<Src>;
export declare function nonnullInput<Src>(ofType: InputType<Src | null>): InputType<Src>;
export declare function queryType<RootSrc>({ name, fields, }: {
    name?: string;
    fields: () => [Field<RootSrc, any, {}>, ...Field<RootSrc, any, {}>[]];
}): ObjectType<RootSrc>;
export declare function mutationType<RootSrc>({ name, fields, }: {
    name?: string;
    fields: () => [Field<RootSrc, any, {}>, ...Field<RootSrc, any, {}>[]];
}): ObjectType<RootSrc>;
export declare function subscriptionField<RootSrc, Out, Arg>({ name, type, args, subscribe, description, deprecationReason, }: {
    name: string;
    type: OutputType<Out>;
    args?: ArgMap<Arg>;
    description?: string;
    deprecationReason?: string;
    subscribe: (src: RootSrc, args: TOfArgMap<ArgMap<Arg>>, ctx: Context, info: graphql.GraphQLResolveInfo) => PromiseOrValue<AsyncIterableIterator<Out>>;
}): SubscriptionField<RootSrc, Arg, Out>;
export declare function subscriptionType<Src>({ name, fields, }: {
    name?: string;
    fields: () => [
        SubscriptionField<Src, any, any>,
        ...SubscriptionField<Src, any, any>[]
    ];
}): SubscriptionObject<Src>;
export {};
