import { l as interfaceType, j as abstractField, p as nonnull, a as id, h as field, d as arg, q as nonnullInput, o as objectType, b as boolean, s as string, m as list, i as int } from './define-90a3cf11.js';
import 'graphql';

function nodeDefinitions(idFetcher) {
    const nodeInterface = interfaceType({
        name: 'Node',
        description: 'An object with an ID',
        fields: () => [
            abstractField({
                name: 'id',
                type: nonnull(id),
                description: 'The id of the object.',
            }),
        ],
    });
    const nodeField = field({
        name: 'node',
        type: nodeInterface,
        args: {
            id: arg({ type: nonnullInput(id), description: 'The ID of an object' }),
        },
        resolve: (_, { id }, context, info) => idFetcher(id, context, info),
    });
    return { nodeInterface, nodeField };
}
const forwardConnectionArgs = {
    after: arg({ type: string }),
    first: arg({ type: int }),
};
const backwardConnectionArgs = {
    before: arg({ type: string }),
    last: arg({ type: int }),
};
const connectionArgs = {
    ...forwardConnectionArgs,
    ...backwardConnectionArgs,
};
const pageInfoType = objectType({
    name: 'PageInfo',
    description: 'Information about pagination in a connection.',
    fields: () => [
        field({
            name: 'hasNextPage',
            type: nonnull(boolean),
            description: 'When paginating forwards, are there more items?',
        }),
        field({
            name: 'hasPreviousPage',
            type: nonnull(boolean),
            description: 'When paginating backwards, are there more items?',
        }),
        field({
            name: 'startCursor',
            type: string,
            description: 'When paginating backwards, the cursor to continue.',
        }),
        field({
            name: 'endCursor',
            type: string,
            description: 'When paginating forwards, the cursor to continue.',
        }),
    ],
});
/**
 * Returns ObjectTypes for a connection with the given name,
 * and whose nodes are of the specified type.
 */
function connectionDefinitions(config) {
    const { nodeType } = config;
    const name = config.name || nodeType.name;
    // TODO
    const edgeFields = config.edgeFields || (() => []);
    const connectionFields = config.connectionFields || (() => []);
    const edgeType = objectType({
        name: name + 'Edge',
        description: 'An edge in a connection.',
        fields: () => [
            field({
                name: 'node',
                type: nonnull(nodeType),
                description: 'The item at the end of the edge',
            }),
            field({
                name: 'cursor',
                type: nonnull(string),
                description: 'A cursor for use in pagination',
            }),
            ...edgeFields(),
        ],
    });
    const connectionType = objectType({
        name: name + 'Connection',
        description: 'A connection to a list of items.',
        fields: () => [
            field({
                name: 'pageInfo',
                type: nonnull(pageInfoType),
                description: 'Information to aid in pagination.',
            }),
            field({
                name: 'edges',
                type: list(edgeType),
                description: 'A list of edges.',
            }),
            ...connectionFields(),
        ],
    });
    return { edgeType, connectionType };
}

export { connectionArgs, connectionDefinitions, nodeDefinitions };
